-- Copyright (C) 1999 Oliver Elphick and others
-- Licensed under Eiffel Forum Freeware License, version 1;
-- (see forum.txt)
--
indexing

   description: "EGTK_MESSAGE - ancestor of all message dialogs"
   author: "Oliver Elphick"
   date: "May 1999"
   cvs: "$Id: egtk_message.e,v 1.9 2003/01/11 19:32:51 elphick Exp $"

deferred class EGTK_MESSAGE

   -- A dialog box containing a message and a close button

inherit

   GTK_DIALOG
      rename
			make as make_dialog,
			dispose as widget_dispose
      end

	GTK_RC
		undefine
			copy, is_equal
		end

feature -- Creation

   make(w: GTK_WINDOW; s: STRING) is
			-- create the dialog box, with w as parent.
			-- s is the message
      require
			good_parent: w /= Void
			good_message: s /= Void and then s.count /= 0
      local
			g: GDK_PIXMAP
			f: STRING
      do
			make_dialog
			set_pathname
			create hbox1.make_horizontal(false, 10)
			f := rc_find_pixmap_in_path (pathname)
			create g.make_from_xpm(w.gdk_window, Void, f)
			create pixmap.make(g, default_pointer)
			hbox1.pack(pixmap, true, true, 5)
			create message.make(s)
			hbox1.pack(message, true, true, 5)
			vbox.pack(hbox1, true, true, 5)
			make_button_box
			Current.reparent(w)
			realize
			create close_cmd.make(Current)
			set_close_cmd(close_cmd)
      ensure
			parent_set: parent = w
      end

   make_with_command(w: GTK_WINDOW; s: STRING; c: EGTK_MESSAGE_HIDE_CMD) is
			-- create the dialog box, with w as parent.
			-- s is the message;
			-- c is the command to run when the close button is clicked
      require
			command_set: c /= Void
      do
			make(w, s)
			set_close_cmd(c)
      end

feature -- Attributes

   hbox1: GTK_BOX

   pixmap: GTK_PIXMAP

   message: GTK_LABEL

   hbox2: GTK_BOX

   close_button: GTK_BUTTON

   pathname: STRING
			-- path to .xpm file

feature -- Procedures

   set_pathname is
			-- set the path to the pixmap
      deferred
      ensure
			path_set: pathname /= Void and then pathname.count /= 0
      end

   set_close_cmd(cmd: EGTK_MESSAGE_HIDE_CMD) is
			-- replace all the Close button callbacks with cmd
      do
			close_button.remove_all_signal_handlers
			add_close_cmd(cmd)
      end

   add_close_cmd(cmd: GTK_COMMAND_TYPE) is
			-- add cmd to the Close button callbacks
      do
			close_button.add_action("clicked", cmd)
      end

   make_button_box is
			-- Create the control button_box
      do
			create hbox2.make_horizontal(true, 30)
			make_buttons
			action_area.pack(hbox2, true, true, 15)
      end

   make_buttons is
			-- Create the control button(s)
			-- redefine this in descendants
      do
			make_close_button
      ensure
			close_button: close_button /= Void and then close_button.parent = hbox2
      end

   make_close_button is
      local
			h: EGTK_MESSAGE_HIDE_CMD
      do
			create close_button.make_with_label("Close")
			hbox2.pack(close_button, false, false, 5)
			create h.make(Current)
			set_close_cmd(h)
      end

   dispose is
      do
			destroy
			widget_dispose
      end

   set_text(s: STRING) is
      require
			not_void: s /= Void
      do
			message.set_text(fold(s))
      end

   set_close_button_text(s: STRING) is
      require
			string_set: s /= Void and then not s.is_empty
			close_button_made: close_button /= Void
      do
			close_button.set_label(s)
      end

   set_close_button_default is
      require
			close_button_made: close_button /= Void
      do
			close_button.set_label("Close")
      end

   close_cmd: EGTK_MESSAGE_HIDE_CMD

   fold(s: STRING): STRING is
		-- fold a string to lines of about 100 characters
      require
			not_void: s /= Void
      local
			lines: ARRAY[STRING]
			x: STRING
			i, j, k: INTEGER
      do
			if s.count > 100 then
				-- split into existing lines (terminated by newlines)
				from
					create lines.make(0,-1)
					x := clone(s)
					i := x.first_index_of('%N')
				variant
					x.occurrences('%N') + 1
				until
					i = 0
				loop
					lines.add_last(x.substring(1,i-1))
					x.remove_head(i)
					i := x.first_index_of('%N')
				end
				lines.add_last(x)
				-- fold each line
				from -- outer loop
					i := lines.lower
					create Result.make_empty
				variant
					lines.count - i + 1
				until
					i > lines.upper
				loop
					if not Result.is_empty then
						Result.extend('%N')
					end
					if (lines @ i).count < 100 then
						Result.append(lines @ i)
					else
						-- this line needs folding
						from -- middle loop
							x := lines @ i
						variant
							x.count + 1
						until
							x.is_empty
						loop
							-- look for last space before position 101
							j := x.reverse_index_of(' ', 100)
							if j = 0 then
								-- No? then next space after that
								k := x.upper
								j := x.substring(101, k).first_index_of(' ')
								if j > 110 then
									-- that is too far; look for a separator
									from -- inner loop
										j := 80
									variant
										x.upper - j
									until
										(x @ j).is_separator or j > 109
									loop
										j := j + 1
									end -- inner loop
									if not (x @ j).is_separator then
										-- No good.  Hyphenate it at 100.
										j := 100
									end
								end
							end -- if
							if j = 0 then j := 100 end
							Result.append(x.substring(1, j-1))
							x.shrink(j, x.upper)
							if x @ 1 = ' ' then
								x.remove(1)
							elseif (x @ j).is_separator then
								Result.extend(x @ j)
								x.remove(1)
							else
								Result.extend('-')
							end
							Result.extend('%N')
						end -- middle loop
					end -- if
					i := i + 1
				end -- outer loop
			else
				Result := s
			end
      end
      

end -- EGTK_MESSAGE
