-- Copyright 2003 Oliver Elphick and others
-- Licensed under Eiffel Forum Freeware License, version 1;
-- (see forum.txt)
--
indexing

   description: "GTK_DRAG - drag and drop features"
   version: "v 0.3.5 GTK+ 1.2.x"
   author: "Oliver Elphick"
   changed: "11 Feb 2003"
   cvs: "$Id$"

deferred class GTK_DRAG
   -- drag and drop features; to be inherited by GTK_WIDGET
   
inherit

   GTK_DND_EXTERNALS
   
feature

   widget: POINTER is
      -- implemented in GTK_WIDGET
      deferred
      end
   
   set_drag_source(mask: INTEGER;
		   gte: GTK_TARGET_ENTRY_LIST;
		   action: INTEGER) is
	 -- make the calling widget a potential source for a drag
      do
	 gtk_drag_source_set(widget, mask, gte.list, gte.count,  action)
      end

   unset_drag_source is
	 -- stop this widget from being a potential drag source
      do
	 gtk_drag_source_unset(widget)
      end
   
   set_drag_destination(mask: INTEGER;
			gte: GTK_TARGET_ENTRY_LIST;
			action: INTEGER) is
	 -- make the calling widget a potential destination for a drag
      do
	 gtk_drag_dest_set(widget, mask, gte.list, gte.count,  action)
      end

   unset_drag_dest is
	 -- stop this widget from being a potential drag destination
      do
	 gtk_drag_dest_unset(widget)
      end

   finish_drag(context: GTK_DRAG_CONTEXT; 
	       success, del: BOOLEAN; tm: INTEGER) is
	 -- Inform the drag source that the drop is finished, and that the 
	 -- data of the drag will no longer be required.
	 -- context : the drag context.
	 -- success : a flag indicating whether the drop was succesful
	 -- del : a flag indicating whether the source should delete 
	 -- the original data. (This should be TRUE for a move)
	 -- tm : the timestamp from the "drag_data_drop" signal
      do
	 gtk_drag_finish(context.context, success, del, tm)
      end

   drag_source_widget(context: GTK_DRAG_CONTEXT): GTK_WIDGET is
      -- the widget where the drag started
      local
	 p: POINTER
      do
	 p := gtk_drag_get_source_widget (context.context)
	 create Result.make_from_pointer(p)
      end

   drag_highlight is
      -- highlight this widget
      do
	 gtk_drag_highlight(widget)
      end

   drag_unhighlight is
      -- unhighlight this widget
      do
	 gtk_drag_unhighlight(widget)
      end

end -- GTK_DRAG
